@@ -12,6 +12,7 @@ import os
 import socket
 import sys
 import time
+import typing as t
 import uuid
 import warnings
 from datetime import datetime
@@ -37,7 +38,8 @@ from IPython.core.error import StdinNotImplementedError
 from jupyter_client.session import Session
 from tornado import ioloop
 from tornado.queues import Queue, QueueEmpty
-from traitlets import (
+from traitlets.config.configurable import SingletonConfigurable
+from traitlets.traitlets import (
     Any,
     Bool,
     Dict,
@@ -50,7 +52,6 @@ from traitlets import (
     default,
     observe,
 )
-from traitlets.config.configurable import SingletonConfigurable
 from zmq.eventloop.zmqstream import ZMQStream
 
 from ipykernel.jsonutil import json_clean
@@ -94,6 +95,10 @@ class Kernel(SingletonConfigurable):
         """
     )
 
+    implementation: str
+    implementation_version: str
+    banner: str
+
     @default("shell_streams")
     def _shell_streams_default(self):
         warnings.warn(
@@ -130,7 +135,7 @@ class Kernel(SingletonConfigurable):
     iopub_socket = Any()
     iopub_thread = Any()
     stdin_socket = Any()
-    log = Instance(logging.Logger, allow_none=True)
+    log: logging.Logger = Instance(logging.Logger, allow_none=True)  # type:ignore[assignment]
 
     # identities:
     int_id = Integer(-1)
@@ -142,7 +147,7 @@ class Kernel(SingletonConfigurable):
 
     # This should be overridden by wrapper kernels that implement any real
     # language.
-    language_info = {}
+    language_info: t.Dict[str, object] = {}
 
     # any links that should go in the help menu
     help_links = List()
@@ -262,7 +267,7 @@ class Kernel(SingletonConfigurable):
         for msg_type in self.control_msg_types:
             self.control_handlers[msg_type] = getattr(self, msg_type)
 
-        self.control_queue = Queue()
+        self.control_queue: Queue[t.Any] = Queue()
 
     def dispatch_control(self, msg):
         self.control_queue.put_nowait(msg)
@@ -278,6 +283,7 @@ class Kernel(SingletonConfigurable):
 
     async def _flush_control_queue(self):
         """Flush the control queue, wait for processing of any pending messages"""
+        tracer_future: t.Union[concurrent.futures.Future[object], asyncio.Future[object]]
         if self.control_thread:
             control_loop = self.control_thread.io_loop
             # concurrent.futures.Futures are threadsafe
@@ -529,7 +535,7 @@ class Kernel(SingletonConfigurable):
     def start(self):
         """register dispatchers for streams"""
         self.io_loop = ioloop.IOLoop.current()
-        self.msg_queue = Queue()
+        self.msg_queue: Queue[t.Any] = Queue()
         self.io_loop.add_callback(self.dispatch_queue)
 
         self.control_stream.on_recv(self.dispatch_control, copy=False)
@@ -749,7 +755,14 @@ class Kernel(SingletonConfigurable):
             self._abort_queues()
 
     def do_execute(
-        self, code, silent, store_history=True, user_expressions=None, allow_stdin=False
+        self,
+        code,
+        silent,
+        store_history=True,
+        user_expressions=None,
+        allow_stdin=False,
+        *,
+        cell_id=None,
     ):
         """Execute user code. Must be overridden by subclasses."""
         raise NotImplementedError
@@ -857,7 +870,7 @@ class Kernel(SingletonConfigurable):
         if hasattr(self, "comm_manager"):
             comms = {
                 k: dict(target_name=v.target_name)
-                for (k, v) in self.comm_manager.comms.items()
+                for (k, v) in self.comm_manager.comms.items()  # type:ignore[attr-defined]
                 if v.target_name == target_name or target_name is None
             }
         else:
@@ -955,7 +968,7 @@ class Kernel(SingletonConfigurable):
             return None
 
     async def usage_request(self, stream, ident, parent):
-        reply_content = {"hostname": socket.gethostname()}
+        reply_content = {"hostname": socket.gethostname(), "pid": os.getpid()}
         current_process = psutil.Process()
         all_processes = [current_process] + current_process.children(recursive=True)
         process_metric_value = self.get_process_metric_value
